#ifndef GARBAGE_PERCENT_ENCODING_HPP
#define GARBAGE_PERCENT_ENCODING_HPP

#include <array>
#include <cstdint>
#include <span>
#include <string>
#include <vector>

namespace Garbage::Percent
{
    namespace Internal
    {
        static constexpr auto EncodeLUT = std::array {
            '0', '1', '2', '3',
            '4', '5', '6', '7',
            '8', '9', 'A', 'B',
            'C', 'D', 'E', 'F',
        };

        [[nodiscard]] constexpr
        std::uint8_t DecodeNibble(char ch) {
            if (ch >= '0' && ch <= '9') {
                return ch - '0';
            }
            if (ch >= 'A' && ch <= 'F') {
                return ch - 'A' + 10;
            }
            if (ch >= 'a' && ch <= 'f') {
                return ch - 'a' + 10;
            }
            return 0;
        }

        [[nodiscard]] constexpr
        std::uint8_t DecodeChar(char upper, char lower) {
            return (DecodeNibble(upper) << 4) | DecodeNibble(lower);
        }

        [[nodiscard]] constexpr
        bool IsHex(char ch) {
            return (ch >= '0' && ch <= '9') || (ch >= 'A' && ch <= 'F') || (ch >= 'a' && ch <= 'f');
        }
    }

    [[nodiscard]] constexpr
    std::string Encode(std::span<std::uint8_t> data)
    {
        std::string result;
        result.reserve(data.size() * 3);

        for (std::uint8_t byte : data) {
            result.push_back('%');
            result.push_back(Internal::EncodeLUT[byte >> 4]);
            result.push_back(Internal::EncodeLUT[byte & 15]);
        }

        return result;
    }

    [[nodiscard]] constexpr
    std::vector<std::uint8_t> Decode(std::string_view data)
    {
        std::vector<std::uint8_t> result;
        result.reserve(data.size() / 3);

        for (std::size_t i = 0; i < data.size(); i += 3) {
            result.push_back(Internal::DecodeChar(data[i + 1], data[i + 2]));
        }

        return result;
    }

    [[nodiscard]] constexpr
    bool Validate(std::string_view data)
    {
        if (data.size() % 3) {
            return false;
        }

        for (std::size_t i = 0; i < data.size(); i += 3) {
            if (data[i] != '%' || !Internal::IsHex(data[i + 1]) || !Internal::IsHex(data[i + 2])) {
                return false;
            }
        }

        return true;
    }
}

#endif // GARBAGE_PERCENT_ENCODING_HPP
